import time

from langchain.document_loaders import UnstructuredMarkdownLoader
from langchain.schema import Document
from langchain.text_splitter import RecursiveCharacterTextSplitter

import re
import argparse

from transwarp_hippo_api.hippo_client import HippoClient
from transwarp_hippo_langchain.hippo_langchain import Hippo

from embedding_factory import EmbeddingFactory



def init_default_env(ip: str, port: str, table_name: str, database_name: str, embedding_model: str, path: str,
                     is_delete: bool = False):
    # 删除table
    if is_delete:
        delete_table_if_exist(ip, port, table_name, database_name)

    init_from_docx(ip, port, table_name, database_name, embedding_model, path)


def delete_table_if_exist(ip: str, port: str, table_name: str, database_name: str):
    cluster_str = ip + ":" + port
    hc = HippoClient([cluster_str])
    try:
        hc.delete_table(table_name, database_name)
        print("delete table[" + table_name + "] success!")
    except Exception as e:
        print("table[" + table_name + "] not exist, ignore delete!")


def init_from_docx(ip: str, port: str, table_name: str, database_name: str, embedding_model: str, path: str):
    loader = UnstructuredMarkdownLoader(path)
    docs = loader.load()
    page_content = re.sub("[\n\r\t]", "", docs[0].page_content)
    meta = docs[0].metadata
    meta['title'] = meta.get("source")
    docs = [Document(page_content=page_content, metadata=meta)]

    # Split the documents into smaller chunks
    text_splitter = RecursiveCharacterTextSplitter(chunk_size=6000, chunk_overlap=100, length_function=len)
    docs = text_splitter.split_documents(docs)
    # 选取模型
    embeddings = EmbeddingFactory().create(embedding_model)
    DEFAULT_HIPPO_CONNECTION = {
        "host": ip,
        "port": port,
    }
    print("input...")
    # 创建vector_store
    Hippo.from_documents(
        docs,
        embedding=embeddings,
        table_name=table_name,
        database_name=database_name,
        connection_args=DEFAULT_HIPPO_CONNECTION
    )
    print("success")


def main():
    parser = argparse.ArgumentParser(description='Process some arguments.')
    parser.add_argument('--ip', type=str, required=True, help='The IP address of the server.')
    parser.add_argument('--port', type=str, required=True, help='The port number to connect to on the server.')
    parser.add_argument('--table_name', type=str, required=True, help='Table name')
    parser.add_argument('--database_name', type=str, required=True, help='Database name')
    parser.add_argument('--embedding_model', type=str, required=True, help='File path')
    parser.add_argument('--file_path', type=str, required=True, help='File path')
    parser.add_argument('--is_delete', type=bool, required=False,
                        help='Whether to delete the table if it exists. Set to True for deletion, False otherwise.')

    args = parser.parse_args()

    init_default_env(args.ip, args.port, args.table_name, args.database_name, args.embedding_model, args.file_path,
                     args.is_delete)


if __name__ == '__main__':
    main()
