import argparse

from langchain import PromptTemplate, LLMChain
from langchain.memory import ConversationTokenBufferMemory
from langchain.schema import Document
from transwarp_hippo_langchain.hippo_langchain import Hippo

from embedding_factory import EmbeddingFactory
from llm_factory import LLMFactory
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

def conversion(ip: str, port: str, table_name: str, database_name: str, embedding_model: str, llm_model:str):
    template = """
            Assistant is a large language model trained by OpenAI to address the inquiries of humans. The 'Article:&Content&' refers to the knowledge provided to you in the form of an article, while the 'Question:&Question&' pertains to the question I pose to you. If there was no 'Content' provided in the last session, you can disregard the 'Question' and answer based on all the previous 'Content'.
            However, if 'Content' was provided in the last session, you need to follow these steps to think and respond within a single session. First, analyze and understand all of the 'Content', then look at the 'Question' and determine its intent. If the 'Question' isn't clear, you can ask the user to clarify.
            Afterwards, I expect you to answer my 'Question' based on the 'Content' I've provided. If the 'Content' can't answer the 'Question', you need to output 'Unable to obtain relevant knowledge from the document', then proceed to answer using your own knowledge. Of course, if you believe the 'Question' has no relation to the 'Content', you can answer according to your own understanding.
            In the end,please answer with chinese
            {history}
            Human: {human_input}
            Assistant:"""


    prompt = PromptTemplate(input_variables=["history", "human_input"], template=template)

    llm = LLMFactory().create(llm_model)

    chatgpt_chain = LLMChain(
        llm=llm,
        prompt=prompt,
        verbose=False,
        memory=ConversationTokenBufferMemory(llm=llm, max_token_limit=8000),
    )

    # 创建embedding model
    embeddings = EmbeddingFactory().create(embedding_model)

    # 链接向量数据库
    connection = {
        "host": ip,
        "port": port,
    }


    vector_store = Hippo.from_documents(
        [Document(page_content="", metadata={})],
        embedding=embeddings,
        table_name=table_name,
        database_name=database_name,
        connection_args=connection
    )

    # 进行会话
    count = 0
    while True:

        if count == 0:
            print(chatgpt_chain.predict(human_input="你好"))
            count += 1

        query = input("\nEnter a query: ")
        print(f"question:{query}")
        if query == "exit":
            break
        if query.strip() == "":
            continue

        else:
            # 创建检索器 （获取最相似的一篇文章）
            res = vector_store.similarity_search(query, 1)
            # 从知识库中检索 获取相似内容
            content_list = [item.page_content for item in res]

            # 接下来，用 join() 函数将这些内容拼接起来
            text = ''.join(content_list)

            # print(f"检索到相似的文章内容为："
            #       f"{text}")

            prompt = f"""
                     Article:&{text}&
                     Question:&{query}&
                     """
            result = chatgpt_chain.predict(human_input=prompt)
            print(f"AI的回答：{result}")

def main():
    parser = argparse.ArgumentParser(description='Process some arguments.')
    parser.add_argument('--ip', type=str, required=True, help='The IP address of the server.')
    parser.add_argument('--port', type=str, required=True, help='The port number to connect to on the server.')
    parser.add_argument('--table_name', type=str, required=True, help='Table name')
    parser.add_argument('--database_name', type=str, required=True, help='Database name')
    parser.add_argument('--embedding_model', type=str, required=True, help='embedding_model')
    parser.add_argument('--llm_name', type=str, required=True, help='llm_name')

    args = parser.parse_args()

    conversion(args.ip, args.port, args.table_name, args.database_name, args.embedding_model, args.llm_name)


if __name__ == '__main__':
    main()
